/*! Copyright (C) [2010] [David Mainzer]
 * 
 *  This program is free software; you can redistribute it and/or modify it under the terms 
 *  of the GNU General Public License as published by the Free Software Foundation; either 
 *  version 3 of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU General Public License for more details.
 *  
 *	You should have received a copy of the GNU General Public License along with this program; 
 *	if not, see <http://www.gnu.org/licenses/>.  
 */

public class Particles
{
	private float lifetime; 		 			//!< total lifetime of the particle
	private float decay;             			//!< decay speed of the particle
	private float [] color = new float[3];    	//!< color values of the particle
	private float [] position = new float[3]; 	//!< position of the particle
	private float [] speed = new float[3]; 		//!< speed of the particle
	
	private boolean active;			 			//!< set particle to active or not

	public final float maxLifetime = 10000F; 	//!< maximal lifetime of particles
	
	Particles()
	{
		lifetime = getNewLifetime();
		decay = getNewDecay();
		color[0] = 0.7F;
		color[1] = 0.0F;
		color[2] = 0.0F;

		position[0] = 0.0F;
		position[1] = 0.0F;
		position[2] = 0.0F;
		
		speed[0] = getNewXSpeed();
		speed[1] = getNewYSpeed();
		speed[2] = getNewZSpeed();

		active = true;
	}

	/*! \brief reset all values to default
	 * 
	 */
	public void setAllNew()
	{
		lifetime = getNewLifetime();
		decay = getNewDecay();
		color[0] = 0.7F;
		color[1] = 0.0F;
		color[2] = 0.0F;

		position[0] = 0.0F;
		position[1] = 0.0F;
		position[2] = 0.0F;
		
		speed[0] = getNewXSpeed();
		speed[1] = getNewYSpeed();
		speed[2] = getNewZSpeed();
		
		active = true;
	}
	
	
	/*! \brief create a new value for lifetime
	 * 
	 *  @return new value
	 */
	public float getNewLifetime()
	{
		return (float) ( (Math.random() * maxLifetime ) / maxLifetime );
	}
	
	/*! \brief create a new value for decay
	 * 
	 *  @return new value
	 */
	public float getNewDecay()
	{
		return (0.035F - (float) ( (Math.random() * 150 ) / 10000.0F ));
	}
	
	/*! \brief create a new value for x-speed
	 * 
	 *  @return new value
	 */
	public float getNewXSpeed()
	{
		return (0.01F - (float) ((Math.random()*200)/10000.0));
	}
	
	/*! \brief create a new value for y-speed
	 * 
	 *  @return new value
	 */
	public float getNewYSpeed()
	{
		return (0.025F - (float) ((Math.random()*100)/10000.0)) ;
	}
	
	/*! \brief create a new value for z-speed
	 * 
	 *  @return new value
	 */
	public float getNewZSpeed()
	{
		return  (0.01F - (float) ((Math.random()*200)/10000.0));
	}
	
	
///////////////////////
// GETTER and SETTER //
///////////////////////	
	
	public float getLifetime()
	{
		return lifetime;
	}

	public void setLifetime(float lifetime)
	{
		this.lifetime = lifetime;
	}

/////////////////////////////////////////////////////////
	
	public float getDecay()
	{
		return decay;
	}
	
	public void setDecay(float decay)
	{
		this.decay = decay;
	}
	
///////////////////////////////////////////////////////////
	public float[] getColor()
	{
		return color;
	}
	
	public float getColorR()
	{
		return color[0];
	}
	
	public void setColorR(float r)
	{
		this.color[0] = r;
	}
	
	public float getColorG()
	{
		return color[1];
	}
	
	public void setColorG(float g)
	{
		this.color[1] = g;
	}
	
	public float getColorB() 
	{
		return color[2];
	}

	public void setColorB(float b) 
	{
		this.color[2] = b;
	}

/////////////////////////////////////////////////////////
	
	public float [] getPos() 
	{
		return position;
	}
	
	public void setPos(float x, float y, float z) 
	{
		this.position[0] = x;
		this.position[1] = y;
		this.position[2] = z;
	}
	
	public float getPosX() 
	{
		return position[0];
	}

	public void setPosX(float xpos) 
	{
		this.position[0] = xpos;
	}
	
	public float getPosY() 
	{
		return position[1];
	}

	public void setPosY(float ypos) 
	{
		this.position[1] = ypos;
	}

	public float getPosZ() 
	{
		return position[2];
	}

	public void setPosZ(float zpos) 
	{
		this.position[2] = zpos;
	}
	
///////////////////////////////////////////////////////////
	
	public float [] getSpeed()
	{
		return speed;
	}
	
	public float getSpeedX()
	{
		return speed[0];
	}

	public void setSpeedX(float xspeed) 
	{
		this.speed[0] = xspeed;
	}
	
	public float getYspeed() 
	{
		return speed[1];
	}

	public void setSpeedY(float yspeed) 
	{
		this.speed[1] = yspeed;
	}
	
	public float getZspeed() 
	{
		return speed[2];
	}

	public void setSpeedZ(float zspeed) 
	{
		this.speed[2] = zspeed;
	}
	
///////////////////////////////////////////////////////////
	
	public boolean getActive ()
	{
		return active;
	}
	
	public void setActive (boolean active)
	{
		this.active = active;
	}
}
